@@ -19,10 +19,10 @@ MSG_CLIENTREQUEST, MSG_CLIENTREPLY, MSG_INCCLIENTREQUEST, \
 FLD_ID, FLD_TYPE, FLD_SRC, FLD_BALLOTNUMBER, FLD_COMMANDNUMBER, \
 FLD_PROPOSAL, FLD_DECISIONS, \
 FLD_REPLY, FLD_REPLYCODE, FLD_INRESPONSETO, FLD_SNAPSHOT, FLD_PVALUESET, FLD_LEADER, \
-FLD_TOKEN, FLD_CLIENTBATCH, FLD_SERVERBATCH, FLD_SENDCOUNT = range(17)
+FLD_TOKEN, FLD_CLIENTBATCH, FLD_SERVERBATCH, FLD_SENDCOUNT, FLD_DECISIONBALLOTNUMBER = range(18)
 
 # node types
-NODE_CLIENT, NODE_ACCEPTOR, NODE_REPLICA, NODE_NAMESERVER = range(4)
+NODE_CLIENT, NODE_REPLICA, NODE_NAMESERVER = range(3)
 
 # error_types
 ERR_NOERROR, ERR_NOTLEADER, ERR_INITIALIZING = range(3)
@@ -55,8 +55,9 @@ TESTTIMEOUT = 1
 BOOTSTRAPCONNECTTIMEOUT = 60
 
 # ballot
-BALLOTNO = 0
-BALLOTNODE = 1
+BALLOTEPOCH = 0
+BALLOTNO = 1
+BALLOTNODE = 2
 
 # backoff
 BACKOFFINCREASE = 0.1
