@@ -29,6 +29,9 @@ def parse_basic(msg):
     src = Peer(msg[FLD_SRC][0], msg[FLD_SRC][1], msg[FLD_SRC][2])
     return Message(msg[FLD_ID], msg[FLD_TYPE], src)
 
+def parse_status(msg):
+    return Message(msg[FLD_ID], msg[FLD_TYPE], msg[FLD_SRC])
+
 def parse_heloreply(msg):
     src = Peer(msg[FLD_SRC][0], msg[FLD_SRC][1], msg[FLD_SRC][2])
     return HeloReplyMessage(msg[FLD_ID], msg[FLD_TYPE],
@@ -99,14 +102,19 @@ def parse_perform(msg):
             pclient = Peer(p[0][0], p[0][1], p[0][2])
             proposal.proposals.append(Proposal(pclient, p[1], p[2]))
     elif msg[FLD_CLIENTBATCH]:
-        proposalclient = Peer(msg[FLD_PROPOSAL][0][0], msg[FLD_PROPOSAL][0][1], msg[FLD_PROPOSAL][0][2])
+        proposalclient = Peer(msg[FLD_PROPOSAL][0][0],
+                              msg[FLD_PROPOSAL][0][1],
+                              msg[FLD_PROPOSAL][0][2])
         proposal = ProposalClientBatch(proposalclient, msg[FLD_PROPOSAL][1], msg[FLD_PROPOSAL][2])
     else:
-        proposalclient = Peer(msg[FLD_PROPOSAL][0][0], msg[FLD_PROPOSAL][0][1], msg[FLD_PROPOSAL][0][2])
+        proposalclient = Peer(msg[FLD_PROPOSAL][0][0],
+                              msg[FLD_PROPOSAL][0][1],
+                              msg[FLD_PROPOSAL][0][2])
         proposal = Proposal(proposalclient, msg[FLD_PROPOSAL][1], msg[FLD_PROPOSAL][2])
     return PerformMessage(msg[FLD_ID], msg[FLD_TYPE], src,
                           msg[FLD_COMMANDNUMBER], proposal,
-                          msg[FLD_SERVERBATCH], msg[FLD_CLIENTBATCH])
+                          msg[FLD_SERVERBATCH], msg[FLD_CLIENTBATCH],
+                          msg[FLD_DECISIONBALLOTNUMBER])
 
 def parse_response(msg):
     src = Peer(msg[FLD_SRC][0], msg[FLD_SRC][1], msg[FLD_SRC][2])
@@ -165,7 +173,7 @@ parse_functions = [
     parse_response, # MSG_RESPONSE
 
     parse_garbagecollect,  # MSG_GARBAGECOLLECT
-    parse_basic,           # MSG_STATUS
+    parse_status,           # MSG_STATUS
     parse_basic            # MSG_ISSUE
     ]
 
