@@ -11,12 +11,13 @@ import time, socket, select
 from Queue import Queue
 from threading import Thread, RLock, Lock, Condition, Timer, Semaphore
 from concoord.enums import *
+from concoord.nameserver import Nameserver
 from concoord.exception import ConnectionError
 from concoord.utils import *
 from concoord.message import *
 from concoord.pack import *
 from concoord.pvalue import PValueSet
-from concoord.connection import ConnectionPool,Connection
+from concoord.connection import ConnectionPool, Connection, SelfConnection
 
 try:
     import dns.resolver, dns.exception
@@ -26,7 +27,7 @@ except:
 parser = argparse.ArgumentParser()
 
 parser.add_argument("-a", "--addr", action="store", dest="addr",
-                    help="addr for the node")
+                    help="address for the node")
 parser.add_argument("-p", "--port", action="store", dest="port", type=int,
                     help="port for the node")
 parser.add_argument("-b", "--boot", action="store", dest="bootstrap",
@@ -35,14 +36,10 @@ parser.add_argument("-o", "--objectname", action="store", dest="objectname", def
                     help="client object dotted name")
 parser.add_argument("-l", "--logger", action="store", dest="logger", default='',
                     help="logger address")
-parser.add_argument("-c", "--configpath", action="store", dest="configpath", default='',
-                    help="config file path")
-parser.add_argument("-n", "--name", action="store", dest="domain", default='',
-                    help="domainname that the nameserver will accept queries for")
-parser.add_argument("-t", "--type", action="store", dest="type", default='',
-                    help="1: Master Nameserver 2: Slave Nameserver (requires a Master) 3:Route53 (requires a Route53 zone)")
-parser.add_argument("-m", "--master", action="store", dest="master", default='',
-                    help="ipaddr:port for the master nameserver")
+parser.add_argument("-n", "--domainname", action="store", dest="domain", default='',
+                    help="domain name that the nameserver will accept queries for")
+parser.add_argument("-r", "--route53", action="store_true", dest="route53", default=False,
+                    help="use Route53 (requires a Route53 zone)")
 parser.add_argument("-w", "--writetodisk", action="store_true", dest="writetodisk", default=False,
                     help="writing to disk on/off")
 parser.add_argument("-d", "--debug", action="store_true", dest="debug", default=False,
@@ -51,34 +48,33 @@ args = parser.parse_args()
 
 class Node():
     """Node encloses the basic Node behaviour and state that
-    are extended by Leaders, Acceptors or Replicas.
+    are extended by Replicas and Nameservers.
     """
     def __init__(self,
-                 nodetype,
                  addr=args.addr,
                  port=args.port,
                  givenbootstraplist=args.bootstrap,
                  debugoption=args.debug,
                  objectname=args.objectname,
-                 instantiateobj=False,
-                 configpath=args.configpath,
                  logger=args.logger,
                  writetodisk=args.writetodisk):
         self.addr = addr if addr else findOwnIP()
         self.port = port
-        self.type = nodetype
         self.debug = debugoption
         self.durable = writetodisk
-        if instantiateobj:
-            if objectname == '':
-                parser.print_help()
-                self._graceexit(1)
-            self.objectname = objectname
+        self.isnameserver = args.domain != ''
+        self.domain = args.domain
+        self.useroute53 = args.route53
+        if objectname == '':
+            parser.print_help()
+            self._graceexit(1)
+        self.objectname = objectname
         # initialize receive queue
         self.receivedmessages_semaphore = Semaphore(0)
         self.receivedmessages = []
         # lock to synchronize message handling
         self.lock = Lock()
+
         # create server socket and bind to a port
         self.socket = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
         self.socket.setsockopt(socket.SOL_SOCKET, socket.SO_REUSEADDR, 1)
@@ -99,50 +95,59 @@ class Node():
                     break
                 except socket.error as e:
                     print "Socket Error: ", e
-                    pass
         self.socket.listen(10)
         self.connectionpool = ConnectionPool()
+
         try:
             self.connectionpool.epoll = select.epoll()
         except AttributeError as e:
             # the os doesn't support epoll
             self.connectionpool.epoll = None
+
+        # set the logger
+        if logger:
+            LOGGERNODE = logger
+        else:
+            LOGGERNODE = None
+
+        # Initialize replicas
+        # Keeps {peer:outofreachcount}
+        self.replicas = {}
+        # Nameserver state
+        if self.isnameserver:
+            self.type = NODE_NAMESERVER
+            try:
+                self.nameserver = Nameserver(self.addr, self.domain, self.useroute53,
+                                             self.replicas, self.debug)
+            except Exception as e:
+                print "Error:", e
+                print "Could not start Replica as a Nameserver, exiting."
+                self._graceexit(1)
+        else:
+            self.type = NODE_REPLICA
+
         self.alive = True
         self.me = Peer(self.addr,self.port,self.type)
         # set id
         self.id = '%s:%d' % (self.addr, self.port)
-
-        # set path for additional configuration data
-        self.configpath = configpath
-        # set the logger
-        try:
-            LOGGERNODE = load_configdict(self.configpath)['LOGGERNODE']
-        except KeyError as e:
-            if logger:
-                LOGGERNODE=logger
-            else:
-                LOGGERNODE = None
+        # add self to connectionpool
+        self.connectionpool.add_connection_to_self(self.me,
+                                                   SelfConnection(self.receivedmessages,
+                                                                  self.receivedmessages_semaphore))
         self.logger = Logger("%s-%s" % (node_names[self.type],self.id), lognode=LOGGERNODE)
+        if self.isnameserver:
+            self.nameserver.add_logger(self.logger)
         print "%s-%s connected." % (node_names[self.type],self.id)
-        # Initialize groups
-        # Keeps {peer:outofreachcount}
-        self.replicas = {}
-        self.acceptors = {}
-        self.nameservers = {}
-        self.groups = {NODE_REPLICA: self.replicas,
-                       NODE_ACCEPTOR: self.acceptors,
-                       NODE_NAMESERVER: self.nameservers}
-        self.groups[self.me.type][self.me] = 0
 
         # Keeps the liveness of the nodes
         self.nodeliveness = {}
+        self.bootstrapset = set()
         # connect to the bootstrap node
         if givenbootstraplist:
-            self.bootstraplist = []
             self.discoverbootstrap(givenbootstraplist)
             self.connecttobootstrap()
-        if self.type == NODE_REPLICA or self.type == NODE_NAMESERVER:
-            self.stateuptodate = False
+
+        self.stateuptodate = False
 
     def _getipportpairs(self, bootaddr, bootport):
         for node in socket.getaddrinfo(bootaddr, bootport, socket.AF_INET, socket.SOCK_STREAM):
@@ -155,7 +160,7 @@ class Node():
             if bootstrap.find(":") >= 0:
                 bootaddr,bootport = bootstrap.split(":")
                 for peer in self._getipportpairs(bootaddr, int(bootport)):
-                    self.bootstraplist.append(peer)
+                    self.bootstrapset.add(peer)
             #dnsname given as bootstrap
             else:
                 answers = []
@@ -165,13 +170,13 @@ class Node():
                     if self.debug: self.logger.write("DNS Error", "Cannot resolve %s" % str(bootstrap))
                 for rdata in answers:
                     for peer in self._getipportpairs(str(rdata.target), rdata.port):
-                        self.bootstraplist.append(peer)
+                        self.bootstrapset.append(peer)
 
     def connecttobootstrap(self):
         tries = 0
         keeptrying = True
         while tries < BOOTSTRAPCONNECTTIMEOUT and keeptrying:
-            for bootpeer in self.bootstraplist:
+            for bootpeer in self.bootstrapset:
                 try:
                     if self.debug: self.logger.write("State",
                                                      "trying to connect to bootstrap: %s" % str(bootpeer))
@@ -215,43 +220,41 @@ class Node():
 
     def statestr(self):
         returnstr = ""
-        for type,group in self.groups.iteritems():
-            for peer in group.iterkeys():
-                returnstr += node_names[type] + " %s:%d\n" % (peer.addr,peer.port)
+        for peer in self.replicas:
+            returnstr += node_names[peer.type] + " %s:%d\n" % (peer.addr,peer.port)
         if hasattr(self, 'pendingcommands') and len(self.pendingcommands) > 0:
-            pending =  "".join("%d: %s" % (cno, proposal) for cno,proposal in self.pendingcommands.iteritems())
+            pending =  "".join("%d: %s" % (cno, proposal) for cno,proposal \
+                                   in self.pendingcommands.iteritems())
             returnstr = "%s\nPending:\n%s" % (returnstr, pending)
         return returnstr
 
     def ping_neighbor(self):
         """used to ping neighbors periodically"""
-        # Only ping neighbors that didn't send a message in less than
-        # LIVENESSTIMEOUT
+        # Only ping neighbors that didn't send a message recently
         while True:
             # Check nodeliveness
-            for gtype,group in self.groups.iteritems():
-                for peer in group:
-                    if peer == self.me:
-                        continue
-                    if peer in self.nodeliveness:
-                        nosound = time.time() - self.nodeliveness[peer]
-                    else:
-                        # This node never sent a message, we should ping it
-                        if self.debug: self.logger.write("State", "Sending PING to %s" % str(peer))
-                        pingmessage = create_message(MSG_PING, self.me)
-                        self.send(pingmessage, peer=peer)
-                        continue
+            for peer in self.replicas:
+                if peer == self.me:
+                    continue
+                if peer in self.nodeliveness:
+                    nosound = time.time() - self.nodeliveness[peer]
+                else:
+                    nosound = LIVENESSTIMEOUT + 1
 
-                    if (4*LIVENESSTIMEOUT) > nosound and nosound > LIVENESSTIMEOUT:
-                        # Send PING to node
-                        if self.debug: self.logger.write("State", "Sending PING to %s" % str(peer))
-                        pingmessage = create_message(MSG_PING, self.me)
-                        self.send(pingmessage, peer=peer)
-                    elif nosound > (4*LIVENESSTIMEOUT):
-                        # Neighbor not responding, mark the neighbor
-                        if self.debug: self.logger.write("State",
-                                                         "Neighbor not responding")
-                        self.groups[peer.type][peer] += 1
+                if nosound <= LIVENESSTIMEOUT:
+                    # Peer is alive
+                    self.replicas[peer] = 0
+                    continue
+                if nosound > LIVENESSTIMEOUT:
+                    # Send PING to node
+                    if self.debug: self.logger.write("State", "Sending PING to %s" % str(peer))
+                    pingmessage = create_message(MSG_PING, self.me)
+                    successid = self.send(pingmessage, peer=peer)
+                if successid < 0 or nosound > (2*LIVENESSTIMEOUT):
+                    # Neighbor not responding, mark the neighbor
+                    if self.debug: self.logger.write("State",
+                                                     "Neighbor not responding")
+                    self.replicas[peer] += 1
             time.sleep(LIVENESSTIMEOUT)
 
     def clean_nascent(self):
@@ -324,7 +327,7 @@ class Node():
                     if s == self.socket:
                         clientsock,clientaddr = self.socket.accept()
                         if self.debug: self.logger.write("State",
-                                                         "accepted a connection from address %s" 
+                                                         "accepted a connection from address %s"
                                                          % str(clientaddr))
                         self.connectionpool.activesockets.add(clientsock)
                         self.connectionpool.nascentsockets.add(clientsock)
@@ -342,17 +345,20 @@ class Node():
         try:
             for message in connection.received_bytes():
                 if self.debug: self.logger.write("State", "received %s" % str(message))
-                # Update self.nodeliveness
-                self.nodeliveness[message.source] = time.time()
                 if message.type == MSG_STATUS:
-                    if self.type == NODE_REPLICA:
                         if self.debug: self.logger.write("State",
-                                                         "Answering status message %s" 
+                                                         "Answering status message %s"
                                                          % self.__str__())
                         messagestr = pickle.dumps(self.__str__())
                         message = struct.pack("I", len(messagestr)) + messagestr
                         clientsock.send(message)
                         return False
+                # Update nodeliveness
+                if message.source.type != NODE_CLIENT:
+                    self.nodeliveness[message.source] = time.time()
+                    if message.source in self.replicas:
+                        self.replicas[message.source] = 0
+
                 # add to receivedmessages
                 self.receivedmessages.append((message,connection))
                 self.receivedmessages_semaphore.release()
@@ -415,19 +421,14 @@ class Node():
 
     def msg_heloreply(self, conn, msg):
         if msg.leader:
-            if msg.source == msg.leader:
-                if self.debug: self.logger.write("Error", "There are no acceptors yet, waiting.")
-                # If heloreply received, should ping the bootstrap again to try connecting.
-                time.sleep(1)
-                self.connecttobootstrap()
-                return
-            elif msg.leader == self.me:
+            if msg.leader == self.me:
                 if self.debug: self.logger.write("State", "I'm the leader.")
                 return
             else:
                 if self.debug: self.logger.write("State", "Adding new bootstrap.")
-                self.bootstraplist.remove(msg.source)
-                self.bootstraplist.append(msg.leader)
+                if msg.source in self.bootstrapset:
+                    self.bootstrapset.remove(msg.source)
+                self.bootstrapset.add(msg.leader)
                 self.connecttobootstrap()
 
     def msg_ping(self, conn, msg):
@@ -492,15 +493,14 @@ class Node():
         elif group:
             ids = []
             for peer,liveness in group.iteritems():
-                if peer != self.me and liveness == 0:
-                    connection = self.connectionpool.get_connection_by_peer(peer)
-                    if connection == None:
-                        if self.debug: self.logger.write("Connection Error",
-                                                         "Connection for %s cannot be found." % str(peer))
-                        continue
-                    connection.send(message)
-                    ids.append(message[FLD_ID])
-                    message[FLD_ID] = assignuniqueid()
+                connection = self.connectionpool.get_connection_by_peer(peer)
+                if connection == None:
+                    if self.debug: self.logger.write("Connection Error",
+                                                     "Connection for %s cannot be found." % str(peer))
+                    continue
+                connection.send(message)
+                ids.append(message[FLD_ID])
+                message[FLD_ID] = assignuniqueid()
             return ids
 
     def terminate_handler(self, signal, frame):
